﻿using BPA.Common.Enums.CRM;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.Extensions;
using BPA.SaaS.Stock.Api.DTO;
using BPA.SaaS.Stock.Api.DTO.Basic.Auth;
using BPA.SaaS.Stock.Api.DTO.Basic.Claim;
using BPA.SaaS.Stock.Api.Entity.SYS;
using BPA.SaaS.Stock.Api.IService.Auth;
using BPA.SaaS.Stock.Api.Repository.Auth;
using BPA.SaaS.Stock.Api.Service.SimpleJWT;
using Microsoft.AspNetCore.Http;

namespace BPA.SaaS.Stock.Api.Service.Auth
{
    /// <summary>
    /// 登录授权相关
    /// </summary>
    /// <returns></returns>
    public class AuthService : IAuthService
    {
        private readonly IHttpContextAccessor _httpContextAccessor;
        // TODO: 不可以这样访问仓储层
        private readonly AuthRepository _authRepository;

        /// <summary>
        /// 
        /// </summary>
        public AuthService(IHttpContextAccessor httpContextAccessor, AuthRepository authRepository)
        {
            _httpContextAccessor = httpContextAccessor;
            _authRepository = authRepository;
        }

        #region 用户

        /// <summary>
        /// 用户名密码登录
        /// </summary>
        /// <param name="LoginType">1平台用户登录,0加盟商登录</param>
        /// <param name="input"></param>
        /// <returns></returns>
        public async Task<BaseResult<LoginOutInfo>> LoginAsync(string LoginType, LoginInput input)
        {
            // 获取加密后的密码
            var encryptPassword = input.Password.ToLower().Md5();

            // 判断用户名和密码是否正确 忽略全局过滤器
            var user = await _authRepository.DbClient.Queryable<BPA_Users>().Where(u => u.Account.Equals(input.Account)
                                                                                        && u.Password.Equals(encryptPassword)
                                                                                        && u.IsDeleted == false
                                                                                        && u.Status == EnumCommonStatus.ENABLE).FirstAsync();
            _ = user ?? throw new Exception(ErrorCode.D1000.GetDescribe());

            //获取权限
            var company = await _authRepository.DbClient.Queryable<BPA_Company>().FirstAsync(x => x.Id == user.GroupId);
            var accessToken = JWTEncryption.Encrypt(new Dictionary<string, object>
            {
                {ClaimConst.CLAINM_USERID, user.Id},
                {ClaimConst.LoginType, LoginType},
                {ClaimConst.CLAINM_ACCOUNT, user.Account},
                {ClaimConst.CLAINM_NAME, user.Name},
                {ClaimConst.CLAINM_SUPERADMIN, user.Account == "admin" ? "1" : "Customer"},
                {ClaimConst.GroupId, user.GroupId},
                {ClaimConst.OrgId, user.SysOrgId},
                {ClaimConst.SupplyPlatformId, company?.SupplyPlatformId}
            });

            // 生成刷新Token令牌
            var refreshToken = JWTEncryption.GenerateRefreshToken(accessToken, 30);

            // 设置刷新Token令牌
            if (_httpContextAccessor.HttpContext != null) _httpContextAccessor.HttpContext.Response.Headers["x-access-token"] = refreshToken;
            var loginOutInfo = new LoginOutInfo
            {
                userID = user.Id,
                token = accessToken
            };
            return BaseResult<LoginOutInfo>.BuildOK(loginOutInfo);
        }

        /// <summary>
        /// 获取当前登录用户信息
        /// </summary>
        /// <returns></returns>
        public async Task<BaseResult<LoginOutput>> GetLoginUserAsync(string logintype)
        {
            var userId = _httpContextAccessor.HttpContext?.User.FindFirst(ClaimConst.CLAINM_USERID)?.Value;

            var user = await _authRepository.DbClient.Queryable<BPA_Users>().Where(u => u.Id == userId).FirstAsync();
            if (user != null)
            {
                return BaseResult<LoginOutput>.BuildOK(new LoginOutput
                {
                    Account = user.Account,
                    Name = user.Name,
                    NickName = user.Name,
                    Avatar = "",
                    isAdmin = user.Account.Equals("admin"),
                    Id = user.Id
                });
            }

            throw new Exception("用户不存在");
        }

        /// <summary>
        /// 退出
        /// </summary>
        /// <returns></returns>
        public BaseResult<string> LogoutAsync() => BaseResult<string>.BuildOK("OK");

        // /// <summary>
        // /// 添加用户
        // /// </summary>
        // /// <param name="input"></param>
        // /// <returns></returns>
        // public bool AddUsers(LoginInput input)
        // {
        //     var flg = db.Queryable<BPA_Users>().First(t => t.Account == input.Account);
        //     if (flg != null)
        //         throw Oops.Oh("当前用户已存在请勿重复添加");
        //     BPA_Users users = new BPA_Users()
        //     {
        //         Id = Guid.NewGuid().ToString(),
        //         Account = input.Account,
        //         CreateAt = DateTime.Now,
        //         // Name = input.Name,
        //         Password = MD5Encryption.Encrypt(input.Password),
        //         //RoleId = input.RoleId,
        //         Status = CommonStatus.ENABLE,
        //         CreateBy = App.User.FindFirst(ClaimConst.CLAINM_NAME).Value
        //     };
        //     return db.Insertable(users).ExecuteCommand() > 0;
        // }
        //
        // /// <summary>
        // /// 重置密码
        // /// </summary>
        // /// <param name="Id"></param>
        // /// <returns></returns>
        // public bool RestorePwd(string Id)
        // {
        //     var Acc = App.User?.FindFirst(ClaimConst.CLAINM_ACCOUNT).Value;
        //     if (string.IsNullOrEmpty(Acc))
        //         throw Oops.Oh("未登录");
        //
        //     var md5 = MD5Encryption.Encrypt("123456");
        //     return db.Updateable<BPA_Users>(t => t.Password == md5).Where(t => t.Id == Id).ExecuteCommandHasChange();
        // }
        //
        // /// <summary>
        // /// 用户页面分页
        // /// </summary>
        // /// <param name="input"></param>
        // /// <returns></returns>
        // public PageUtil UserPage(LoginPageInput input)
        // {
        //     int total = 0;
        //     var data = db.Queryable<BPA_Users, BPA_Roles>((t, x) => new JoinQueryInfos(JoinType.Left, t.Id == x.Id))
        //         .Where(t => t.Account.ToLower() != "admin")
        //         .WhereIF(!string.IsNullOrEmpty(input.Name), t => t.Name.Contains(input.Name))
        //         .Select((t, x) => new LoginOutput
        //         {
        //             Id = t.Id,
        //             Account = t.Account,
        //             Name = t.Name,
        //             Stutas = (int) t.Status,
        //             RoleName = x.Name
        //         }).ToPageList(input.PageIndex, input.PageSize, ref total);
        //     return new PageUtil
        //     {
        //         Data = data,
        //         Total = total
        //     };
        // }
        //
        // /// <summary>
        // /// 修改用户密码
        // /// </summary>
        // /// <param name="input"></param>
        // /// <returns></returns>
        // public bool AlterPwd(UserInputDTO input)
        // {
        //     var Old = MD5Encryption.Encrypt(input.OldPwd);
        //     var New = MD5Encryption.Encrypt(input.NewPwd);
        //     BPA_Users User = db.Queryable<BPA_Users>().Where(t => t.Id == input.Id && t.Password == Old).First();
        //     if (User == null) throw Oops.Oh("旧密码错误");
        //     return db.Updateable<BPA_Users>(t => t.Password == New).Where(t => t.Id == input.Id).ExecuteCommandHasChange();
        // }
        //
        // /// <summary>
        // /// 启用禁用
        // /// </summary>
        // /// <param name="Id"></param>
        // /// <param name="type"></param>
        // /// <returns></returns>
        // public bool AlterUserStatus(string Id, bool type)
        // {
        //     return db.Updateable<BPA_Users>().SetColumnsIF(type == true, t => t.Status == CommonStatus.ENABLE)
        //         .SetColumnsIF(type == false, t => t.Status == CommonStatus.DISABLE)
        //         .Where(t => t.Id == Id).ExecuteCommandHasChange();
        // }
        //
        // /// <summary>
        // /// 删除用户
        // /// </summary>
        // /// <param name="Id"></param>
        // /// <returns></returns>
        // public bool RemovUser(string Id)
        // {
        //     return db.Deleteable<BPA_Users>(t => t.Id == Id).ExecuteCommandHasChange();
        // }
        //
        // /// <summary>
        // /// 变更用户权限
        // /// </summary>
        // /// <param name="Id"></param>
        // /// <param name="RoleId"></param>
        // /// <returns></returns>
        // public bool AlterUserRole(string Id, string RoleId)
        // {
        //     return db.Updateable<BPA_Users>().SetColumns(t => t.Id == RoleId).Where(t => t.Id == Id).ExecuteCommandHasChange();
        // }

        #endregion
    }
}